import dotenv from "dotenv"
import { log } from './src/Controllers/LogController.js'

function loadEnvironment() {
    const knownEnv = {
        'dev': '.env.dev',
        'prod': '.env.prod'
    }
    const args = process.argv.slice(2) // Get the command-line arguments
    
    if (args.length !== 1 || !knownEnv[args]) {
        log({ type: 'error', message: 'Usage: node index.js [dev|prod]' })
        process.exit(1)
    }
    
    // Choose the appropriate .env file based on the argument passed
    const envFilePath = './config/' + knownEnv[args]

    log({ type: 'info', message: envFilePath })
    
    // Load the environment variables from the chosen .env file
    const result = dotenv.config({ path: envFilePath })
    
    if (result.error) {
        log({ type: 'error', message: `Error loading ${envFilePath}: ${result.error.message}` })
        process.exit(1)
    }
}

export default loadEnvironment
