import MenuRepository from '../Repositories/MenuRepository.js'
import moment from 'moment'

const getMenuRestaurants = async function ({ consDetails }) {
    const restaurants = await MenuRepository.getMenuRestaurants(consDetails)

    return restaurants
}

const buildMenu = async function ({ CDCLIENTE, CDFILIAL, CDCAIXA, CDLOJA, NRCONFTELA, groups }) {
    const productPriceTable = await buscaTabelaPadrao(CDFILIAL, CDCLIENTE, CDLOJA)
    const menu = await montaCardapio(CDFILIAL, CDCLIENTE, CDLOJA, NRCONFTELA, productPriceTable.precosIndexadosPorProduto, groups)
    return menu
}

const getRestaurantGroups = async function ({ CDFILIAL, CDLOJA, NRCONFTELA }) {
    const restaurantGroups = await MenuRepository.gruposProduto(CDFILIAL, CDLOJA, NRCONFTELA)
    return restaurantGroups
}

const buscaTabelaPadrao = async function (CDFILIAL, CDCLIENTE, CDLOJA) {
    const tabelaPadrao = await buscaTabelaVigente(CDFILIAL, CDCLIENTE, CDLOJA)
    if (tabelaPadrao.error == false) {
        const precos = await MenuRepository.oraclePrecos(tabelaPadrao.CDFILIAL, tabelaPadrao.CDTABEPREC, tabelaPadrao.DTINIVGPREC)

        const precosDia = await buscaPrecosDia(tabelaPadrao.CDFILIAL, tabelaPadrao.CDTABEPREC, tabelaPadrao.DTINIVGPREC)

        if (precos.length > 0) {
            var precosIndexadosPorProduto = []
            precos.forEach((preco) => {
                precosIndexadosPorProduto[preco['CDPRODUTO']] = preco
            })

            if (precosDia.length > 0) {
                precosDia.forEach((precoDia) => {
                    const codProdutoDia = precoDia['CDPRODUTO']

                    if (precosIndexadosPorProduto[codProdutoDia] && precosIndexadosPorProduto[codProdutoDia]['VRPRECITEM'] !== null) {
                        var precoFinal = 0
                        if (precoDia['IDPERVALORPR'] == 'V') {
                            if (precoDia['IDDESCACREPR'] == 'D') {
                                //Desconto por valor
                                precoFinal = $precosIndexadosPorProduto[$codProdutoDia]['VRPRECITEM'] - precoDia['VRPRECODIA'];
                            } else {
                                //Acrescimo por valor
                                precoFinal = $precosIndexadosPorProduto[$codProdutoDia]['VRPRECITEM'] + precoDia['VRPRECODIA'];
                            }
                        } else {
                            if (precoDia['IDDESCACREPR'] == 'D') {
                                //Desconto por porcentagem
                                precoFinal = $precosIndexadosPorProduto[$codProdutoDia]['VRPRECITEM'] * (1 - precoDia['VRPRECODIA']/100);
                            } else {
                                //Acrescimo por porcentagem
                                precoFinal = $precosIndexadosPorProduto[$codProdutoDia]['VRPRECITEM'] * (1 + (precoDia['VRPRECODIA']/100));
                            }
                        }

                        if (precoFinal > 0) {
                            $precosIndexadosPorProduto[$codProdutoDia]['VRPRECITEM'] = precoFinal;
                        }
                    }
                })
            }

            return {
                error: false,
                precosIndexadosPorProduto: precosIndexadosPorProduto
            }
        } else {
            return {
                error: true,
                message: 'Não há produtos para a tabela de preços vigente.'
            }
        }
    }
    else {
        return tabelaPadrao
    }
}

const buscaTabelaVigente = async function (CDFILIAL, CDCLIENTE, CDLOJA) {
    // busca tabela de preço por CLIENTEFILIAL
    const tabelaClienteFilial = await MenuRepository.clienteFilial(CDFILIAL, CDCLIENTE)

    if (tabelaClienteFilial[0]['CDTABEPREC']) {
        // valida a vigência
        const DTINIVGPREC = await validaVigencia(tabelaClienteFilial[0]['CDCFILTABPRE'], tabelaClienteFilial[0]['CDTABEPREC'])
        if (DTINIVGPREC == false) {
            return {
                error: true,
                message: 'Tabela de preços associada ao cliente/filial está inativa.'
            }
        } else {
            return {
                error: false,
                CDFILIAL: tabelaClienteFilial[0]['CDCFILTABPRE'],
                CDTABEPREC: tabelaClienteFilial[0]['CDTABEPREC'],
                DTINIVGPREC: DTINIVGPREC
            }
        }


    } else {
        // busca a tabela de preço por CLIENTE
        const tabelaCliente = await MenuRepository.precoCliente(CDCLIENTE)
        if (tabelaCliente[0]['CDTABEPREC']) {
            // valida a vigência
            const DTINIVGPREC = await validaVigencia(tabelaCliente[0]['CDFILTABPREC'], tabelaCliente[0]['CDTABEPREC'])
            if (DTINIVGPREC == false) {
                return {
                    error: true,
                    message: 'Tabela de preços associada ao cliente está inativa.'
                }
            } else {
                return {
                    error: false,
                    CDFILIAL: tabelaCliente[0]['CDFILTABPREC'],
                    CDTABEPREC: tabelaCliente[0]['CDTABEPREC'],
                    DTINIVGPREC: DTINIVGPREC
                }
            }
        } else {
            // busca tabela de preco por LOJA
            const tabelaLoja = await MenuRepository.precoLoja(CDFILIAL, CDLOJA)
            if (tabelaLoja[0]['CDTABEPREC']) {
                // valida a vigência
                const DTINIVGPREC = await validaVigencia(CDFILIAL, tabelaLoja[0]['CDTABEPREC'])
                if (DTINIVGPREC == false) {
                    return {
                        error: true,
                        message: 'Tabela de preços associada à loja está inativa.'
                    }
                } else {
                    return {
                        error: false,
                        CDFILIAL: CDFILIAL,
                        CDTABEPREC: tabelaLoja[0]['CDTABEPREC'],
                        DTINIVGPREC: DTINIVGPREC
                    }
                }
            } else {
                // busca tabela de preço por FILIAL (padrão)
                const paravend = await MenuRepository.paravend(CDFILIAL)
                if (paravend[0]['CDTABEPREC'] == false) {
                    return {
                        error: true,
                        message: 'Não há tabela de preços cadastrada.'
                    }
                } else {
                    // valida a vigência
                    const DTINIVGPREC = await validaVigencia(CDFILIAL, paravend[0]['CDTABEPREC'])
                    if (DTINIVGPREC == false) {
                        return {
                            error: true,
                            message: 'Tabela de preços está inativa.'
                        }
                    } else {
                        return {
                            error: false,
                            CDFILIAL: CDFILIAL,
                            CDTABEPREC: paravend[0]['CDTABEPREC'],
                            DTINIVGPREC: DTINIVGPREC
                        }
                    }
                }
            }
        }
    }
}

const validaVigencia = async function (CDFILIAL, CDTABEPREC) {
    const result = await MenuRepository.tabelaVendaOracle(CDFILIAL, CDTABEPREC)
    if (result) {
        return result['DTINIVGPREC']
    } else {
        return false
    }
}

const buscaPrecosDia = async function (CDFILIAL, CDTABEPREC, DTINIVGPREC) {
    
    const dataAtual = new Date()
    const hora = dataAtual.getHours() + dataAtual.getMinutes() + dataAtual.getSeconds()
    
    // Expected: 1 (for Sunday) through 7 (for Saturday)
    const dia = dataAtual.getDay() // 0 - 6
    const diaSemana = dia + 1
    
    const precosDia = await MenuRepository.oraclePrecosDia(CDFILIAL, CDTABEPREC, DTINIVGPREC, hora, diaSemana, 'T')

    return precosDia
}

const montaCardapio = async function (CDFILIAL, CDCLIENTE, CDLOJA, NRCONFTELA, precosIndexadosPorProduto, groups, CDCONSUMIDOR = null) {
    const grupoProdutos = groups

    if (grupoProdutos.length > 0) {
        const subGrupoProdutos = await MenuRepository.subGrupoProdutos(CDFILIAL, NRCONFTELA)
        const NRDIASEMANABLOQ = new Date().getDay() + 1

        const produtos = await MenuRepository.getProdutos(CDFILIAL, CDLOJA, NRCONFTELA, NRDIASEMANABLOQ)
        // const observacoesProduto = await MenuRepository.observacoesProduto(CDFILIAL, CDLOJA, NRDIASEMANABLOQ)
        const observacoesProduto = []
        var observacoesIndexadasPorProduto = []

        observacoesProduto.forEach((obsAtual) => {
            if (!(observacoesIndexadasPorProduto[obsAtual['CDPRODUTO']])) {
                observacoesIndexadasPorProduto[obsAtual['CDPRODUTO']] = []
            }
            observacoesIndexadasPorProduto[obsAtual['CDPRODUTO']].push(obsAtual['CDOCORR'])
        })

        const arrayProdutos = await montaArrayProdutos(grupoProdutos, subGrupoProdutos, produtos, precosIndexadosPorProduto, observacoesIndexadasPorProduto, CDLOJA, CDFILIAL, NRCONFTELA, CDCLIENTE, CDCONSUMIDOR)

        if (!arrayProdutos.error) {
            return {
                error: false,
                cardapio: Object.values(arrayProdutos)
            }
        } else {
            return {
                error: true,
                message: 'Não há grupo de produtos na configuração de tela do caixa.'
            }
        }
    }
}

const montaArrayProdutos = async function (grupoProdutos, subGrupoProdutos, produtos, precosIndexadosPorProduto, observacoesIndexadasPorProduto, CDLOJA, CDFILIAL, NRCONFTELA, CDCLIENTE, CDCONSUMIDOR) {
    var arrayRetorno = []

    // Processa os grupos.
    grupoProdutos.forEach((grupoAtual) => {
        const codGrupo = grupoAtual['NRPGCONFTELA'] + grupoAtual['NRBUTTON']
        const URL_SERVIDOR_IMAGEM = 'http://midia.teknisa.com/files/';
        arrayRetorno[codGrupo] = {
            'grupo': {
                CODIGO: codGrupo,
                DESC: grupoAtual['DSBUTTON'],
                COLOR: grupoAtual['NRCOLORBACK'],
                DSENDEIMG: grupoAtual['DSENDEIMG'] ? URL_SERVIDOR_IMAGEM + grupoAtual['DSENDEIMG'] : null,
                NRBUTTON: grupoAtual['NRBUTTON']
            },
            'produtos': []
        }
    })

    // Processa os subgrupos.
    subGrupoProdutos.forEach((subGrupoAtual) => {
        const codGrupo = subGrupoAtual['NRPGCONFTAUX'] + subGrupoAtual['NRBUTTONAUX']
        const codSubGrupo = subGrupoAtual['NRPGCONFTELA'] + subGrupoAtual['NRBUTTON']

        if (!arrayRetorno[codGrupo]['subgrupos']) {
            arrayRetorno[codGrupo]['subgrupos'] = []
        }

        arrayRetorno[codGrupo]['subgrupos'][codSubGrupo] = {
            'subgrupo': {
                CODIGO: codSubGrupo,
                DESC: subGrupoAtual['DSBUTTON'],
                COLOR: subGrupoAtual['NRCOLORBACK']
            },
            'produtos': []
        }
    })

    // Processa os produtos.
    for (const produtoAtual of produtos) {
        // Define preço do produto.
        var doPreco = 0
        if (precosIndexadosPorProduto && precosIndexadosPorProduto[produtoAtual['CDPRODUTO']]) {
            doPreco = parseFloat(precosIndexadosPorProduto[produtoAtual['CDPRODUTO']]['VRPRECITEM'])
        } else {
            const preco = await buscaPreco(CDFILIAL, CDCLIENTE, produtoAtual['CDPRODUTO'], CDLOJA, CDCONSUMIDOR)
            if (preco['retirarProduto']) {
                produtos.splice(produtoAtual, 1)
            }
            if (preco.error) {
                continue
            } else {
                doPreco = preco.PRECO
            }
        }

        // Define as impressoras do produto.
        const impressorasProduto = null

        // Trata promoção inteligente.
        var gruposPromocao = []
        const URL_SERVIDOR_IMAGEM = 'http://midia.teknisa.com/files/';

        if (produtoAtual && produtoAtual['IDTIPOCOMPPROD'] == '3') {
            const composicaoPadrao = await MenuRepository.gruposPromocaoInteligenteFilial(produtoAtual['CDIDENTBUTON'], CDFILIAL)

            var usaComposicaoPadrao = false
            var gruposComposicao = []
            if (composicaoPadrao.length > 0) {
                usaComposicaoPadrao = false
                gruposComposicao = composicaoPadrao
            } else {
                usaComposicaoPadrao = true
                gruposComposicao = await MenuRepository.gruposPromocaoInteligente(produtoAtual['CDIDENTBUTON'])
            }


            if (gruposComposicao.length > 0) {
                for (const smartGroup of gruposComposicao) {
                    gruposPromocao[smartGroup['CDGRUPROMOC']] = {
                        'grupo': {
                            'CDGRUPROMOC': smartGroup['CDGRUPROMOC'],
                            'NMGRUPROMOC': smartGroup['NMGRUPROMOC'],
                            'QTPRGRUPPROMOC': parseInt(smartGroup['QTPRGRUPPROMOC'])
                        },
                        'produtos': []
                    }

                    const NRDIASEMANABLOQ = new Date().getDay() + 1

                    var produtosComposicao = []
                    if (usaComposicaoPadrao) {
                        produtosComposicao = await MenuRepository.prodPromoInteligenteOracle(produtoAtual['CDIDENTBUTON'], smartGroup['CDGRUPROMOC'], CDFILIAL, CDLOJA, NRDIASEMANABLOQ)
                    } else {
                        produtosComposicao = await MenuRepository.prodPromoInteligenteFilialOracle(produtoAtual['CDIDENTBUTON'], smartGroup['CDGRUPROMOC'], CDFILIAL, CDLOJA, NRDIASEMANABLOQ)
                    }


                    var doPrecoPromoc = ''

                    if (produtosComposicao.length > 0) {
                        for (const produtoComposicaoAtual of produtosComposicao) {
                            if (precosIndexadosPorProduto && precosIndexadosPorProduto[produtoAtual['CDIDENTBUTON']]['VRPRECITEM']) {
                                if (precosIndexadosPorProduto[produtoComposicaoAtual['CDPRODUTO']] && precosIndexadosPorProduto[produtoComposicaoAtual['CDPRODUTO']]['VRPRECITEM']) {
                                    doPrecoPromoc = precosIndexadosPorProduto[produtoComposicaoAtual['CDPRODUTO']]['VRPRECITEM']
                                } else {
                                    doPrecoPromoc = 0
                                }
                            } else {
                                doPrecoPromoc = 0
                            }
                           
                            gruposPromocao[smartGroup['CDGRUPROMOC']]['produtos'][produtoComposicaoAtual['CDPRODUTO']] = {
                                'CDPRODUTO': produtoComposicaoAtual['CDPRODUTO'],
                                'NMPRODUTO': produtoComposicaoAtual['DSAPELIDOMOB'] ? produtoComposicaoAtual['DSAPELIDOMOB'] : produtoComposicaoAtual['NMPRODUTO'],
                                'VRPRECITEM': doPrecoPromoc,
                                'OBSERVACOES': observacoesIndexadasPorProduto && observacoesIndexadasPorProduto[produtoComposicaoAtual['CDPRODUTO']] ? observacoesIndexadasPorProduto[produtoComposicaoAtual['CDPRODUTO']] : [],
                                'DSPRODVENDA': produtoComposicaoAtual['DSPRODVENDA'],
                            }

                            gruposPromocao[smartGroup['CDGRUPROMOC']]['produtos'] = Object.values(gruposPromocao[smartGroup['CDGRUPROMOC']]['produtos'])
                        }
                    }
                }
            }
        }

        // Formatação final
        const produtoConstruido = {
            'CDPRODUTO': produtoAtual['CDIDENTBUTON'],
            'NMPRODUTO': produtoAtual['DSBUTTON'],
            'GRUPOS': Object.values(gruposPromocao),
            'VRPRECITEM': doPreco,
            'DSENDEIMG': produtoAtual['DSENDEIMG'] ? URL_SERVIDOR_IMAGEM + produtoAtual['DSENDEIMG'] : null,
            'DSPRODVENDA': produtoAtual['DSPRODVENDA'],
            'NMGRUPO': arrayRetorno[produtoAtual['NRPGCONFTAUX'] + produtoAtual['NRBUTTONAUX']]['grupo']['DESC'],
        }

        // Insere o produto no grupo ou subgrupo.
        const TIPO_GRUPO = '2';
        const TIPO_SUB_GRUPO = 'I';
        var codGrupo = ''
        if (produtoAtual['IDTPBUTONAUX'] == TIPO_GRUPO) {
            codGrupo = produtoAtual['NRPGCONFTAUX'] + produtoAtual['NRBUTTONAUX']
            arrayRetorno[codGrupo]['produtos'][produtoAtual['CDIDENTBUTON']] = produtoConstruido
        } else if (produtoAtual['IDTPBUTONAUX'] == TIPO_SUB_GRUPO) {
            const codSubGrupo = produtoAtual['NRPGCONFTAUX'] + produtoAtual['NRBUTTONAUX']
            codGrupo = produtoAtual['NRPGCONFTAUX2'] + produtoAtual['NRBUTTONAUX2']
            arrayRetorno[codGrupo]['subgrupos'][codSubGrupo]['produtos'][produtoAtual['CDIDENTBUTON']] = produtoConstruido
        }
    }

    // Busca produtos combinados.
    const NRDIASEMANABLOQ = new Date().getDay() + 1
    const produtosCombo = await MenuRepository.produtosCombo(CDFILIAL, CDLOJA, NRCONFTELA)

    // Processa os produtos combinados.
    if (produtosCombo.length > 0) {
        formattedCombos = []

        var i = 0 // Identificador do produto pai.
        for (const combo of produtosCombo) {
            // Busca a composição do produto combinado.
            const comboChildProducts = await MenuRepository.produtosComboOracle(CDFILIAL, CDLOJA, NRCONFTELA, parseInt(combo['NRBUTTON']), NRDIASEMANABLOQ.toString())

            // Define preços e observações.
            comboChildProducts.forEach((childProduct) => {
                if (precosIndexadosPorProduto && precosIndexadosPorProduto[comboChildProducts['CDIDENTBUTON']]['VRPRECITEM']) {
                    childProduct['VRPRECITEM'] = precosIndexadosPorProduto[comboChildProducts['CDIDENTBUTON']]['VRPRECITEM']
                } else {
                    childProduct['VRPRECITEM'] = 0
                }
                childProduct['OBSERVACOES'] = observacoesIndexadasPorProduto && observacoesIndexadasPorProduto[comboChildProducts['CDIDENTBUTON']] ? observacoesIndexadasPorProduto[comboChildProducts['CDIDENTBUTON']] : []
            })

            // Container necessário do produto combinado.
            const comboCode = i.toString().padStart(5, '0')
            const standardGroup = {
                [comboCode]: {
                    'grupo': {
                        'CDGRUPROMOC': comboCode,
                        'NMGRUPROMOC': '',
                        'QTPRGRUPPROMOC': combo['NRLIMPRODCOM']
                    },
                    'produtos': comboChildProducts // Composição (produtos filhos).
                }
            }

            // Formatação do produto combinado.
            const preparedCombo = {
                'CDARVPROD': combo['CDARVPROD'],
                'CDBARPRODUTO': combo['CDBARPRODUTO'],
                'CDPRODINTE': combo['CDPRODINTE'],
                'CDPRODUTO': comboCode,
                'NMPRODUTO': combo['DSBUTTON'],
                'DTFINVGPROMOC': combo['DTFINVGPROMOC'],
                'DTINIVGPROMOC': combo['DTINIVGPROMOC'],
                'GRUPOS': Object.values(standardGroup), // Itens do produto combinado.
                'OBSERVACOES': [],
                'IMPRESSORAS': [],
                'IDIMPPRODUTO': '2', // Produto pai nunca será impresso.
                'IDPESAPROD': combo['IDPESAPROD'],
                'IDTIPOCOMPPROD': 'C',
                'IDTIPOPROD': combo['IDTIPOPROD'],
                'IDTIPCOBRA': combo['IDTIPCOBRA'],
                'VRPRECITEM': 0.01,
                'IDPRODBLOQ': combo['IDPRODBLOQ'],
                'NRCOLORBACK': combo['NRCOLORBACK'],
                'IDCONTROLAREFIL': combo['IDCONTROLAREFIL'],
                'DSENDEIMG': null,
                'DSPRODVENDA': combo['DSPRODVENDA'],
                'DSADICPROD': null,
                'NMGRUPO': 'COMBOS'
            }
            formattedCombos.push(preparedCombo)
            i++
        }

        // Cria um grupo para os produtos combinados.
        if (formattedCombos.length > 0) {
            arrayRetorno['X'] = {
                'grupo': {
                    'CODIGO': 'X',
                    'DESC': 'COMBOS',
                    'COLOR': '8421376.000',
                    'DSENDEIMG': null,
                    'NRBUTTON': null
                },
                'produtos': formattedCombos
            }
        }
    }

    // Busca todos os grupos promocionais presentes no cardapio com preço atualizado.
    const gruposPromocionais = await buscaGruposPromocionais(arrayRetorno, CDFILIAL, CDCLIENTE, CDLOJA, CDCONSUMIDOR)
    await atualizaPrecoGrupPromoc(arrayRetorno, gruposPromocionais)


    arrayRetorno.forEach((grupoAtual) => {
        if (grupoAtual.produtos)
            grupoAtual.produtos = Object.values(grupoAtual.produtos) // Transforma o objeto em array.
        else if (grupoAtual.subgrupos) {
            grupoAtual.subgrupos.forEach((subgrupoAtual) => {
                subgrupoAtual.produtos = Object.values(subgrupoAtual.produtos) // Transforma o objeto em array.
            })
            grupoAtual.subgrupos = Object.values(grupoAtual.subgrupos) // Transforma o objeto em array.
        }
    })

    return arrayRetorno
}

const buscaPreco = async function (CDFILIAL, CDCLIENTE, CDPRODUTO, CDLOJA, CDCONSUMIDOR) {
    const tabelaEscolhida = await buscaTabelaVigente(CDFILIAL, CDCLIENTE, CDLOJA)
    if (tabelaEscolhida.error == false) {
        return await buscaPrecoProduto(
            tabelaEscolhida.CDFILIAL,
            tabelaEscolhida.CDTABEPREC,
            CDPRODUTO,
            tabelaEscolhida.DTINIVGPREC,
            CDCLIENTE,
            CDCONSUMIDOR
        )
    }
    else {
        return tabelaEscolhida
    }
}  

const buscaPrecoProduto = async function (CDFILIAL, CDTABEPREC, CDPRODUTO, DTINIVGPRECP, CDCLIENTE, CDCONSUMIDOR) {
    var retorno = {
        error: false,
        PRECOCLIE: 0,
        PRECOVAR: 0,
        PRECOSUGER: 0,
        DESC: 0,
        ACRE: 0,
        PRECO: 0,
        retirarProduto: false
    }

    // formata a data de vigência
    const FORMATO_BRASILEIRO = "D/M/Y"
    const DTINIVGPREC = moment(DTINIVGPRECP, "DD-MM-YY hh:mm:ss").format(FORMATO_BRASILEIRO)


    // busca dados do produto
    const itemTabPreco = await MenuRepository.itemPrecoOracle(CDFILIAL, CDTABEPREC, DTINIVGPREC, CDPRODUTO)

    // se não tiver preço cadastrado retorna o código do erro
    if (itemTabPreco.length == 0) {
        retorno.error = true
        retorno.message = 'Preço não cadastrado para o produto ' + CDPRODUTO + '.'
        retorno.retirarProduto = true
    }
    else if (parseFloat(itemTabPreco['PRECO']) + parseFloat(itemTabPreco['PRECOCLIE']) === 0) {
        retorno.error = true
        retorno.message = 'O preço do produto ' + CDPRODUTO + ' está zerado.'
    } else {
        result = await precoDiaF(CDFILIAL, CDTABEPREC, CDPRODUTO, DTINIVGPREC, CDCLIENTE, CDCONSUMIDOR)

        var precoDia = parseFloat(result.VRPRECODIA)
        const perc = result.PERC
        const descAcre = result.DESCACRE
        const idVisuaCupom = result.IDVISUACUPOM

        retorno.PRECOVAR = parseFloat(itemTabPreco['IDPRECVARIA'])
        retorno.PRECOCLIE = parseFloat(itemTabPreco['PRECOCLIE'])
        retorno.PRECOSUGER = parseFloat(itemTabPreco['PRECOSUGER'])

        // verifica se existe preco do dia
        if (precoDia == 0) {
            retorno.PRECO = parseFloat(itemTabPreco['PRECO'])
        } else {
            // acrescenta ou desconta, o valor ou percentual, do preço cadastrado
            retorno.PRECOVAR = parseFloat(itemTabPreco['IDPRECVARIA'])
            retorno.PRECOCLIE = parseFloat(itemTabPreco['PRECOCLIE'])
            retorno.PRECOSUGER = 0

            // define se é valor (V) ou percentual (P)
            if (perc == "V") {
                // define se é desconto (D) ou acréscimo (A)
                if (descAcre == "A") {
                    // define se desconto aparece no cupom ou não
                    if (idVisuaCupom == "N") {
                        retorno.ACRE = 0
                        retorno.PRECO = parseFloat(itemTabPreco['PRECO']) + precoDia
                    } else {
                        retorno.ACRE = precoDia
                        retorno.PRECO = parseFloat(itemTabPreco['PRECO'])
                    }
                } else {
                    // define se desconto aparece no cupom ou não
                    if (idVisuaCupom == "N") {
                        retorno.DESC = 0
                        retorno.PRECO = parseFloat(itemTabPreco['PRECO']) - precoDia
                    } else {
                        retorno.DESC = precoDia
                        retorno.PRECO = parseFloat(itemTabPreco['PRECO'])
                    }
                }
            } else {
                // define se é desconto (D) ou acréscimo (A)

                if (descAcre == "A") {
                    acrescimoAux = (parseFloat(itemTabPreco['PRECO']) * precoDia) / 100
                    // define se desconto aparece no cupom ou não
                    if (idVisuaCupom == "N") {
                        retorno.ACRE = 0
                        retorno.PRECO = parseFloat(itemTabPreco['PRECO']) + acrescimoAux
                    } else {
                        retorno.ACRE = acrescimoAux
                        retorno.PRECO = parseFloat(itemTabPreco['PRECO'])
                    }
                } else {
                    itemTabPreco['PRECO'] = Math.floor(parseFloat(itemTabPreco['PRECO']) * 100) / 100
                    descontoAux = Math.floor((parseFloat(itemTabPreco['PRECO']) * precoDia) / 100 * 100) / 100

                    // define se desconto aparece no cupom ou não
                    if (idVisuaCupom == "N") {
                        retorno.DESC = 0
                        retorno.PRECO = Math.floor((parseFloat(itemTabPreco['PRECO']) - descontoAux) * 100) / 100

                        if (retorno.PRECO < 0.01) {
                            retorno.PRECO = 0.01
                        }
                    } else {
                        retorno.DESC = descontoAux
                        retorno.PRECO = parseFloat(itemTabPreco['PRECO'])
                    }
                }
            }
        }

    }

    return retorno
}

const precoDiaF = async function (CDFILIAL, CDTABEPREC, CDPRODUTO, DTINIVGPREC, CDCLIENTE, CDCONSUMIDOR) {
    const dataAtual = new Date()

    var retorno = await MenuRepository.tipoConsumidor(CDCLIENTE, CDCONSUMIDOR)
    if (retorno) {
        tipoConsumidor = retorno['CDTIPOCONS']
    }
    else {
        tipoConsumidor = "T"
    }

    retorno = await MenuRepository.feriado(CDFILIAL, dataAtual.format("d/m/Y"))
    if (retorno) {
        dia = "F"
    } else {
        dia = dataAtual.getDay() + 1
    }

    const hora = dataAtual.getHours() + dataAtual.getMinutes() + dataAtual.getSeconds()

    var itemPrecoDia = await MenuRepository.itemPrecoDiaOracle(CDFILIAL, CDTABEPREC, DTINIVGPREC, CDPRODUTO, CDPRODUTO, dia.toString(), tipoConsumidor, hora)
    if (itemPrecoDia) {
        retorno.PERC = itemPrecoDia['IDPERVALORPR']
        retorno.DESCACRE = itemPrecoDia['IDDESCACREPR']
        retorno.IDVISUACUPOM = itemPrecoDia['IDVISUACUPOM']
        retorno.VRPRECODIA = itemPrecoDia['VRPRECODIA']
    } else {
        itemPrecoDia = await MenuRepository.itemPrecoDiaOracle(CDFILIAL, CDTABEPREC, DTINIVGPREC, CDPRODUTO, CDPRODUTO, "T", tipoConsumidor, hora)
        if (itemPrecoDia) {
            retorno.PERC = itemPrecoDia['IDPERVALORPR']
            retorno.DESCACRE = itemPrecoDia['IDDESCACREPR']
            retorno.IDVISUACUPOM = itemPrecoDia['IDVISUACUPOM']
            retorno.VRPRECODIA = itemPrecoDia['VRPRECODIA']
        } else {
            dia = (dataAtual.getDay() + 1).toString()
            itemPrecoDia = await MenuRepository.itemPrecoDiaOracle(CDFILIAL, CDTABEPREC, DTINIVGPREC, CDPRODUTO, CDPRODUTO, dia, tipoConsumidor, hora)
            if (itemPrecoDia) {
                retorno.PERC = itemPrecoDia['IDPERVALORPR']
                retorno.DESCACRE = itemPrecoDia['IDDESCACREPR']
                retorno.IDVISUACUPOM = itemPrecoDia['IDVISUACUPOM']
                retorno.VRPRECODIA = itemPrecoDia['VRPRECODIA']
            } else {
                retorno = null
            }
        }
    }

    return retorno
}

const buscaGruposPromocionais = async function (cardapio, CDFILIAL, CDCLIENTE, CDLOJA, CDCONSUMIDOR) {
    var gruposPromocionais = []
    for (const grupoAtual of cardapio) {
        if (grupoAtual && grupoAtual['subgrupos'] && grupoAtual['subgrupos'].length > 0) {
            for (const subGrupoAtual of grupoAtual['subgrupos']) {
                for (const produtoSubGrupoAtual of subGrupoAtual['produtos']) {
                    if (produtoSubGrupoAtual && produtoSubGrupoAtual['IDTIPOCOMPPROD'] == '3') {
                        for (const grupoPromocAtual of produtoSubGrupoAtual['GRUPOS']) {
                            const chaveExiste = gruposPromocionais.includes(grupoPromocAtual['grupo']['CDGRUPROMOC'])
                            if (!chaveExiste) {
                                for (const produtoGrupoPromocAtual of grupoPromocAtual['produtos']) {
                                    const preco = await buscaPreco(CDFILIAL, CDCLIENTE, prodGrupoPromocAtual['CDPRODUTO'], CDLOJA, CDCONSUMIDOR)
                                    prodGrupoPromocAtual['VRPRECITEM'] = preco['PRECO']
                                }
                                gruposPromocionais.push(grupoPromocAtual['grupo']['CDGRUPROMOC'])
                            }
                        }
                    }
                }
            }
        } else {
            if (grupoAtual && grupoAtual['produtos']) {
                for (const produtoAtual of grupoAtual['produtos']) {
                    if (produtoAtual && produtoAtual['IDTIPOCOMPPROD'] == '3') {
                        for (const grupoPromocAtual of produtoAtual['GRUPOS']) {
                            const chaveExiste = gruposPromocionais.includes(grupoPromocAtual['grupo']['CDGRUPROMOC'])
                            if (!chaveExiste) {
                                for (const prodGrupoPromocAtual of grupoPromocAtual['produtos']) {
                                    const preco = await buscaPreco(CDFILIAL, CDCLIENTE, prodGrupoPromocAtual['CDPRODUTO'], CDLOJA, CDCONSUMIDOR)
                                    prodGrupoPromocAtual['VRPRECITEM'] = preco['PRECO']
                                }
                                gruposPromocionais.push(grupoPromocAtual['grupo']['CDGRUPROMOC'])
                            }
                        }
                    }
                }
            }
        }
    }

    return gruposPromocionais
}

const atualizaPrecoGrupPromoc = function (cardapio, gruposPromocionais) {
    cardapio.forEach((grupoAtual) => {
        if (grupoAtual['produtos'].length > 0) {
            grupoAtual['produtos'].forEach((produtoAtual) => {
                if (produtoAtual && produtoAtual['IDTIPOCOMPPROD'] == '3') {
                    // produto é combo
                    produtoAtual['GRUPOS'].forEach((grupoPromocAtual) => {
                        // percorre produtos do grupo promocional e atualiza preços
                        grupoPromocAtual['produtos'].forEach((prodGrupoPromocAtual) => {
                            prodGrupoPromocAtual['VRPRECITEM'] = gruposPromocionais[grupoPromocAtual['grupo']['CDGRUPROMOC']]['produtos'][prodGrupoPromocAtual['CDPRODUTO']]['VRPRECITEM']
                        })
                    })
                }
            })
        }
    })
}

export default { getMenuRestaurants, buildMenu, getRestaurantGroups }
