import RegisterRepository from '../Repositories/RegisterRepository.js'
import BaseInfoController from './BaseInfoController.js'

const getClients = (req, body) => {
    return RegisterRepository.getClients()
}
const getStates = (req, body) => {
    return RegisterRepository.getStates(body)
}
const getCities = (req, body) => {
    return RegisterRepository.getCities(body)
}

const checkConsumerByLogin = async (req, body) => {
    let result = {
        error: false,
        message: ''
    }
    const consumer = await RegisterRepository.getConsumerByLogin(body)
    if (consumer.length > 0) {
        result.error = true
        result.message = `O login ${body.NMLOGINCONS} não está disponível.`
    }
    return result
}

const checkConsumerByCPF = async (req, body) => {
    let result = {
        error: false,
        message: ''
    }
    const consumer = await RegisterRepository.getConsumerByCPF(body)
    if (consumer.length > 0) {
        result.error = true
        result.message = 'O CPF já encontra-se cadastrado'
    }
    return result
}

const checkConsumerById = async (req, body) => {
    let result = {
        error: false,
        message: []
    }

    if (body && body.length > 0) {
        for (let i = 0; i < body.length; i++) {
            const consumer = await RegisterRepository.getConsumerById(body[i].CDIDCONSUMID, body[i].CDCLIENTE)
            if (consumer && consumer.length > 0) {
                let message = `Matricula ${body[i].CDIDCONSUMID} já se encontra cadastrada no sistema`
                result.error = true
                result.message.push(message)
            }
        }
    }
    return result
}

const registerUser = async (req, body) => {
 try {
    const baseInfo = await BaseInfoController.getBaseInfo()
    const NRORG = baseInfo[0].NRORG
    let newConsumer = { ...body }
    delete newConsumer.sonsData
    delete newConsumer.CDFILIAL
    newConsumer.CDTIPOCONS = await getCDTIPOCONS(body.CDFILIAL)
    newConsumer.CDCCUSCLIE = await getCDCCUSCLIE(body.CDFILIAL)
    newConsumer.CDPAIS = '0055'
    newConsumer.IDATUCONSUMI = 'S'
    newConsumer.IDSITCONSUMI = '1' //ativo
    newConsumer.IDCONSUMIDOR = 'C'
    newConsumer.CDVENDEDOR = ''
    newConsumer.IDTPVENDACONS = '7'
    newConsumer.IDIMPCPFCUPOM = 'S'
    newConsumer.IDCADCONFLIBCON = 'N'
    newConsumer.IDPERCONSPRODEX = 'S'
    newConsumer.IDTPSELMANHA = 'N'
    newConsumer.IDTPSEALMOCO = 'N'
    newConsumer.IDTPSELTARDE = 'N'
    newConsumer.IDCRACHAMESTRE = 'S'

    for (let i = 0; i < body.sonsData.length; i++) {
        let paramsNewCode = {
            NRORGNOVOCODIGO: NRORG,
            NOMETABELA: 'CONSUMIDOR' + body.CDCLIENTE
        }
        newConsumer.CDCONSUMIDOR = await RegisterRepository.getNewCode(paramsNewCode)
        newConsumer.NMCONSUMIDOR = body.sonsData[i].NMCONSUMIDOR
        newConsumer.CDIDCONSUMID = body.sonsData[i].CDIDCONSUMID
        await RegisterRepository.registerUser(newConsumer)
    }

    return {
         error: false, 
         message: 'Usuário cadastrado com sucesso'
    }

 } catch (error) {
    return {
        error: true,
        message: 'Falha ao registrar usuário.'
    }
 }
}

function getCDTIPOCONS (CDFILIAL) {
    return RegisterRepository.getCDTIPOCONS(CDFILIAL)
}

function getCDCCUSCLIE (CDFILIAL) {
    return RegisterRepository.getCDCCUSCLIE(CDFILIAL)
}

const addDependents = async (consDetails, body) => {
    try {
        const baseInfo = await BaseInfoController.getBaseInfo()
        const NRORG = baseInfo[0].NRORG
        const consumer = await RegisterRepository.getConsumerByCdConsumidor(consDetails.CDCLIENTE, consDetails.CDCONSUMIDOR)
        const paramsNewCode = {
            NRORGNOVOCODIGO: NRORG,
            NOMETABELA: 'CONSUMIDOR' + body.CDCLIENTE
        }
        let newConsumer = consumer
        newConsumer.IDCRACHAMESTRE = 'N'
        newConsumer.CDCONSUMIDOR = await RegisterRepository.getNewCode(paramsNewCode)
        newConsumer.NMCONSUMIDOR = body.NMCONSUMIDOR
        newConsumer.CDIDCONSUMID = body.CDIDCONSUMID
        newConsumer.IDSITCONSUMI = '1'
        await RegisterRepository.registerUser(newConsumer)
        return {
            error: false, 
            message: 'Novo dependente registrado com sucesso'
        }
    } catch (error) {
        return {
            error: true,
            message: 'Falha ao registrar novo dependente.'
        }
    }
}

export default { getClients, getStates, getCities, registerUser, checkConsumerByLogin, checkConsumerByCPF, checkConsumerById, addDependents }
