import { executeQuery } from '../Connection/QueriesExecutor.js'
import { sanitizeNumber } from './../Libs/Util.js'

const login = async function (body) {
    let result = await executeQuery({ sql: 'GET_CONSUMER_BY_LOGIN',
        rawReturn: true,
        binds: {
            'CDCLIENTE': body.client,
            'TYPEDLOGIN': body.username,
            'CDSENHACONSMD5': body.refreshToken ? body.refreshToken : body.password
        }
    })

    if (result?.length === 0) {
        let sanitizedUsername = sanitizeNumber(body.username)

        if (sanitizedUsername?.length > 0) {
            result = await executeQuery({ sql: 'GET_CONSUMER_BY_LOGIN',
                rawReturn: true,
                binds: {
                    'CDCLIENTE': body.client,
                    'TYPEDLOGIN': sanitizedUsername,
                    'CDSENHACONSMD5': body.refreshToken ? body.refreshToken : body.password
                }
            })
        }
    }
    
    return result
}

const loginClientless = async function (body) {
    // Esse ternário permite, em ambiente de desenvolvimento, com bases de clientes que não sejam Sapore
    // (The Fifties por exemplo), simular as rotinas da Sapore
    const sql = process.env.ENV_TYPE == 'dev' && process.env.usa_instituicao_cliente_filial == 'S'
        ? 'GET_CONSUMER_CLIENTLESS_DEV' : 'GET_CONSUMER_CLIENTLESS'

    let result = await executeQuery({ sql,
        rawReturn: true,
        binds: {
            'TYPEDLOGIN': body.username,
            'CDSENHACONSMD5': body.refreshToken ? body.refreshToken : body.password
        }
    })

    if (result?.length === 0) {
        let sanitizedUsername = sanitizeNumber(body.username)

        if (sanitizedUsername?.length > 0) {
            result = await executeQuery({ sql,
                rawReturn: true,
                binds: {
                    'TYPEDLOGIN': sanitizedUsername,
                    'CDSENHACONSMD5': body.refreshToken ? body.refreshToken : body.password
                }
            })
        }
    }
    
    return result
}

const loginSaporeWithClient = async function (body) {
    // Esse ternário permite, em ambiente de desenvolvimento, com bases de clientes que não sejam Sapore
    // (The Fifties por exemplo), simular as rotinas da Sapore
    const sql = process.env.ENV_TYPE == 'dev' && process.env.usa_instituicao_cliente_filial == 'S'
        ? 'GET_CONSUMER_BY_LOGIN_SAPORE_DEV' : 'GET_CONSUMER_BY_LOGIN_SAPORE'

    let result = await executeQuery({ sql,
        rawReturn: true,
        binds: {
            'CDCLIENTE': body.client,
            'TYPEDLOGIN': body.username,
            'CDSENHACONSMD5': body.refreshToken ? body.refreshToken : body.password
        }
    })

    if (result?.length === 0) {
        let sanitizedUsername = sanitizeNumber(body.username)

        if (sanitizedUsername?.length > 0) {
            result = await executeQuery({ sql,
                rawReturn: true,
                binds: {
                    'CDCLIENTE': body.client,
                    'TYPEDLOGIN': sanitizedUsername,
                    'CDSENHACONSMD5': body.refreshToken ? body.refreshToken : body.password
                }
            })
        }
    }
    
    return result
}

const getClients = async function () {
    let sql = process.env.usa_instituicao_cliente_filial === 'S' ? 'GET_CLIENTS_SAPORE' : 'GET_CLIENTS'

    const result = await executeQuery({ sql,
        rawReturn: true
    })

    return result
}

const getConsumerByLoginRecover = async function (CDCLIENTE, NMLOGINCONS) {
    let result = await executeQuery({ sql: 'GET_CONSUMER_BY_LOGIN_RECOVER',
        rawReturn: true,
        binds: {
            CDCLIENTE,
            TYPEDLOGIN: NMLOGINCONS
        }   
    })

    // Limpa o NMLOGINCONS e tenta buscar novamente no banco se tiver o comprimento de um CPF
    if (result?.length === 0) {
        let sanitizedUsername = sanitizeNumber(NMLOGINCONS)
    
        if (sanitizedUsername?.length === 11) {
            result = await executeQuery({ sql: 'GET_CONSUMER_BY_LOGIN_RECOVER',
                rawReturn: true,
                binds: {
                    CDCLIENTE,
                    TYPEDLOGIN: sanitizedUsername
                }   
            })
        }
    }

    return result
}

const getConsumerClientlessRecover = async function (NMLOGINCONS) {
    // Esse ternário permite, em ambiente de desenvolvimento, com bases de clientes que não sejam Sapore
    // (The Fifties por exemplo), simular as rotinas da Sapore
    const sql = process.env.ENV_TYPE == 'dev' && process.env.usa_instituicao_cliente_filial == 'S'
        ? 'GET_CONSUMER_CLIENTLESS_RECOVER_DEV' : 'GET_CONSUMER_CLIENTLESS_RECOVER'

    let result = await executeQuery({ sql,
        rawReturn: true,
        binds: {
            TYPEDLOGIN: NMLOGINCONS
        }
    })

    // Limpa o NMLOGINCONS e tenta buscar novamente no banco se tiver o comprimento de um CPF
    if (result?.length === 0) {
        let sanitizedUsername = sanitizeNumber(NMLOGINCONS)
    
        if (sanitizedUsername?.length === 11) {
            result = await executeQuery({ sql,
                rawReturn: true,
                binds: {
                    TYPEDLOGIN: sanitizedUsername
                }
            })
        }
    }

    return result
}

const activateUser = async function (data) {
    const { CDCLIENTE, NRCPFRESPCON } = data
    const result = await executeQuery({sql: 'ACCEPT_USE_TERMS', 
        binds: { CDCLIENTE, NRCPFRESPCON }
    })
    return result
}

const blockUser = async function (data) {
    const { CDCLIENTE, NRCPFRESPCON } = data
    const result = await executeQuery({sql: 'DESACTIVATE_USER', 
        binds: { CDCLIENTE, NRCPFRESPCON }
    })
    return result
}

export default {
    login,
    loginClientless,
    loginSaporeWithClient,
    getClients,
    getConsumerByLoginRecover,
    getConsumerClientlessRecover,
    activateUser,
    blockUser
}
